/*
 * LPGPL v2 or later
 * (C) Neil Jagdish Patel <njpatel@gmail.com>
 */

namespace Gwibber
{
  public class StreamView : Gtk.HBox
  {
    private Gtk.Adjustment     adjustment;
    private Gtk.VScrollbar     scrollbar;
    private Gtk.VBox           view_box;

    private Dee.Model? _model = null;
    private uint _position = 0;

    public Dee.Model model {
      get { return _model; }
      set {
        _model = value;
        adjustment.set_upper (_model.get_n_rows ());
        refresh ();
      }
    }

    public uint position {
      get { return _position; }
      set {
        if (_position != value)
          {
            _position = value;
            adjustment.set_value (value);
            refresh ();
          }
      }
    }

    private SList<StreamViewTile> tiles;

    private int last_width = 0;
    private int last_height = 0;

    public StreamView ()
    {
      Object (homogeneous:false, spacing:6);
    }

    ~StreamView ()
    {

    }

    construct
    {
      adjustment = new Gtk.Adjustment (0, 0, 1, 1, 1, 1);
      adjustment.notify["value"].connect (()=> {
        _position = (int)adjustment.value;

        refresh ();
      });
           
      view_box = new Gtk.VBox (false, 8);
      pack_start (view_box, true, true, 0);

      scrollbar = new Gtk.VScrollbar (adjustment);
      pack_start (scrollbar, false, false, 0);

      for (int i = 0; i < 10; i++)
        {
          var tile = new StreamViewTile ();
          tile.show_all ();
          view_box.pack_start (tile, false, false, 0);
          tiles.append (tile);
        }

      size_allocate.connect_after ((a) => {
        if (last_width != a.width
            || last_height != a.height)
          {
            last_width = a.width;
            last_height = a.height;

            refresh ();
          }
      });
    }

    /*
     * This refreshes the view with the latest model and position
     */
    private void refresh ()
    {
      uint i = 0;
      uint n_rows = _model.get_n_rows ();

      if (!(_model is Dee.Model))
        {
          foreach (StreamViewTile tile in tiles)
            tile.reset ();
          return;
        }

      unowned Dee.ModelIter? iter;
      iter = _model.get_iter_at_row (_position);
      
      foreach (StreamViewTile tile in tiles)
        {
          if (iter != null && position + i < n_rows)
            {
              tile.set_details (_position + i,
                                _model.get_string (iter, StreamModelColumn.SERVICE),
                                _model.get_string (iter, StreamModelColumn.NAME),
                                _model.get_uint   (iter, StreamModelColumn.TIMESTAMP),
                                _model.get_string (iter, StreamModelColumn.MESSAGE),
                                _model.get_string (iter, StreamModelColumn.ICON_URI));
            }
          else
            {
              tile.reset ();
            }
          i++;
          if (position + i > n_rows)
            iter = null;
          else
            iter = _model.get_iter_at_row (_position + i);
        }
    }
  }
}
